@@ -4,13 +4,10 @@ use miniquad::*;
 
 pub use miniquad::{FilterMode, ShaderError};
 
-use crate::{color::Color, telemetry, texture::Texture2D};
+use crate::{color::Color, logging::warn, telemetry, texture::Texture2D};
 
 use std::collections::BTreeMap;
 
-const MAX_VERTICES: usize = 10000;
-const MAX_INDICES: usize = 5000;
-
 #[derive(Debug, Clone, Copy, PartialEq)]
 pub enum DrawMode {
     Triangles,
@@ -21,8 +18,8 @@ pub enum DrawMode {
 pub struct GlPipeline(usize);
 
 struct DrawCall {
-    vertices: [Vertex; MAX_VERTICES],
-    indices: [u16; MAX_INDICES],
+    vertices: Vec<Vertex>,
+    indices: Vec<u16>,
 
     vertices_count: usize,
     indices_count: usize,
@@ -36,6 +33,7 @@ struct DrawCall {
     pipeline: GlPipeline,
     uniforms: Option<Vec<u8>>,
     render_pass: Option<RenderPass>,
+    capture: bool,
 }
 
 #[repr(C)]
@@ -100,11 +98,15 @@ impl DrawCall {
         pipeline: GlPipeline,
         uniforms: Option<Vec<u8>>,
         render_pass: Option<RenderPass>,
+        max_vertices: usize,
+        max_indices: usize,
     ) -> DrawCall {
         DrawCall {
-            vertices: [Vertex::new(0., 0., 0., 0., 0., Color::new(0.0, 0.0, 0.0, 0.0));
-                MAX_VERTICES],
-            indices: [0; MAX_INDICES],
+            vertices: vec![
+                Vertex::new(0., 0., 0., 0., 0., Color::new(0.0, 0.0, 0.0, 0.0));
+                max_vertices
+            ],
+            indices: vec![0; max_indices],
             vertices_count: 0,
             indices_count: 0,
             clip: None,
@@ -114,6 +116,7 @@ impl DrawCall {
             pipeline,
             uniforms,
             render_pass,
+            capture: false,
         }
     }
 
@@ -288,6 +291,7 @@ struct GlState {
     snapshotter: MagicSnapshotter,
 
     render_pass: Option<RenderPass>,
+    capture: bool,
 }
 
 impl GlState {
@@ -321,7 +325,7 @@ impl PipelineExt {
              }| uniform_name == name,
         );
         if uniform_meta.is_none() {
-            println!("Trying to set non-existing uniform: {}", name);
+            warn!("Trying to set non-existing uniform: {}", name);
             return;
         }
         let uniform_meta = uniform_meta.unwrap();
@@ -330,7 +334,7 @@ impl PipelineExt {
         let uniform_byte_offset = uniform_meta.byte_offset;
 
         if std::mem::size_of::<T>() != uniform_byte_size {
-            println!(
+            warn!(
                 "Trying to set uniform {} sized {} bytes value of {} bytes",
                 name,
                 std::mem::size_of::<T>(),
@@ -536,6 +540,8 @@ pub struct QuadGl {
     start_time: f64,
 
     white_texture: Texture,
+    max_vertices: usize,
+    max_indices: usize,
 }
 
 impl QuadGl {
@@ -554,6 +560,7 @@ impl QuadGl {
                 depth_test_enable: false,
                 snapshotter: MagicSnapshotter::new(ctx),
                 render_pass: None,
+                capture: false,
             },
             draw_calls: Vec::with_capacity(200),
             draw_calls_bindings: Vec::with_capacity(200),
@@ -561,6 +568,8 @@ impl QuadGl {
             start_time: miniquad::date::now(),
 
             white_texture,
+            max_vertices: 10000,
+            max_indices: 5000,
         }
     }
 
@@ -641,12 +650,12 @@ impl QuadGl {
             let vertex_buffer = Buffer::stream(
                 ctx,
                 BufferType::VertexBuffer,
-                MAX_VERTICES * std::mem::size_of::<Vertex>(),
+                self.max_vertices * std::mem::size_of::<Vertex>(),
             );
             let index_buffer = Buffer::stream(
                 ctx,
                 BufferType::IndexBuffer,
-                MAX_INDICES * std::mem::size_of::<u16>(),
+                self.max_indices * std::mem::size_of::<u16>(),
             );
             let bindings = Bindings {
                 vertex_buffers: vec![vertex_buffer],
@@ -666,7 +675,6 @@ impl QuadGl {
             .iter_mut()
             .zip(self.draw_calls_bindings.iter_mut())
         {
-            telemetry::track_drawcall(dc.vertices_count, dc.indices_count);
             let pipeline = self.pipelines.get_quad_pipeline_mut(dc.pipeline);
 
             let (width, height) = if let Some(render_pass) = dc.render_pass {
@@ -698,6 +706,7 @@ impl QuadGl {
             bindings
                 .images
                 .resize(2 + pipeline.textures.len(), Texture::empty());
+
             for (pos, name) in pipeline.textures.iter().enumerate() {
                 if let Some(texture) = pipeline.textures_data.get(name).copied() {
                     bindings.images[2 + pos] = texture;
@@ -725,16 +734,23 @@ impl QuadGl {
                 pipeline.uniforms_data.len(),
             );
             ctx.draw(0, dc.indices_count as i32, 1);
+            ctx.end_render_pass();
+
+            if dc.capture {
+                telemetry::track_drawcall(&pipeline.pipeline, &bindings, dc.indices_count);
+            }
 
             dc.vertices_count = 0;
             dc.indices_count = 0;
-
-            ctx.end_render_pass();
         }
 
         self.draw_calls_count = 0;
     }
 
+    pub(crate) fn capture(&mut self, capture: bool) {
+        self.state.capture = capture;
+    }
+
     pub fn get_projection_matrix(&self) -> glam::Mat4 {
         // get_projection_matrix is a way plugins used to get macroquad's current projection
         // back in the days when projection was a part of static batcher
@@ -787,8 +803,12 @@ impl QuadGl {
     }
 
     pub fn geometry(&mut self, vertices: &[impl Into<VertexInterop> + Copy], indices: &[u16]) {
-        assert!(vertices.len() <= MAX_VERTICES);
-        assert!(indices.len() <= MAX_INDICES);
+        if vertices.len() >= self.max_vertices || indices.len() >= self.max_indices {
+            warn!("geometry() exceeded max drawcall size, clamping");
+        }
+
+        let vertices = &vertices[0..self.max_vertices.min(vertices.len())];
+        let indices = &indices[0..self.max_indices.min(indices.len())];
 
         let pip = self.state.pipeline.unwrap_or(
             self.pipelines
@@ -809,8 +829,9 @@ impl QuadGl {
                 || draw_call.pipeline != pip
                 || draw_call.render_pass != self.state.render_pass
                 || draw_call.draw_mode != self.state.draw_mode
-                || draw_call.vertices_count >= MAX_VERTICES - vertices.len()
-                || draw_call.indices_count >= MAX_INDICES - indices.len()
+                || draw_call.vertices_count >= self.max_vertices - vertices.len()
+                || draw_call.indices_count >= self.max_indices - indices.len()
+                || draw_call.capture != self.state.capture
                 || self.state.break_batching
         }) {
             let uniforms = self.state.pipeline.map_or(None, |pipeline| {
@@ -830,6 +851,8 @@ impl QuadGl {
                     pip,
                     uniforms.clone(),
                     self.state.render_pass,
+                    self.max_vertices,
+                    self.max_indices,
                 ));
             }
             self.draw_calls[self.draw_calls_count].texture = self.state.texture;
@@ -840,6 +863,7 @@ impl QuadGl {
             self.draw_calls[self.draw_calls_count].model = self.state.model();
             self.draw_calls[self.draw_calls_count].pipeline = pip;
             self.draw_calls[self.draw_calls_count].render_pass = self.state.render_pass;
+            self.draw_calls[self.draw_calls_count].capture = self.state.capture;
 
             self.draw_calls_count += 1;
             self.state.break_batching = false;
@@ -887,6 +911,34 @@ impl QuadGl {
             .entry(name.to_owned())
             .or_insert(texture.texture) = texture.texture;
     }
+
+    pub(crate) fn update_drawcall_capacity(
+        &mut self,
+        ctx: &mut Context,
+        max_vertices: usize,
+        max_indices: usize,
+    ) {
+        self.max_vertices = max_vertices;
+        self.max_indices = max_indices;
+
+        for binding in &mut self.draw_calls_bindings {
+            let vertex_buffer = Buffer::stream(
+                ctx,
+                BufferType::VertexBuffer,
+                self.max_vertices * std::mem::size_of::<Vertex>(),
+            );
+            let index_buffer = Buffer::stream(
+                ctx,
+                BufferType::IndexBuffer,
+                self.max_indices * std::mem::size_of::<u16>(),
+            );
+            *binding = Bindings {
+                vertex_buffers: vec![vertex_buffer],
+                index_buffer,
+                images: vec![Texture::empty(), Texture::empty()],
+            };
+        }
+    }
 }
 
 mod shader {
