@@ -330,10 +330,11 @@ impl Skin {
             },
             tabbar_style: Style {
                 margin: Some(RectOffset::new(2., 2., 2., 2.)),
-                color: Color::from_rgba(204, 204, 204, 235),
-                color_clicked: Color::from_rgba(187, 187, 187, 255),
+                color: Color::from_rgba(220, 220, 220, 235),
+                color_clicked: Color::from_rgba(187, 187, 187, 235),
                 color_hovered: Color::from_rgba(170, 170, 170, 235),
-                color_selected: Color::from_rgba(240, 240, 240, 235),
+                color_selected_hovered: Color::from_rgba(180, 180, 180, 235),
+                color_selected: Color::from_rgba(204, 204, 204, 235),
                 text_color: Color::from_rgba(0, 0, 0, 255),
                 ..Style::default(default_font.clone())
             },
