@@ -23,6 +23,14 @@ pub fn clear_background(color: Color) {
     context.gl.clear(&mut context.quad_context, color);
 }
 
+#[doc(hidden)]
+pub fn gl_set_drawcall_buffer_capacity(max_vertices: usize, max_indices: usize) {
+    let context = get_context();
+    context
+        .gl
+        .update_drawcall_capacity(&mut context.quad_context, max_vertices, max_indices);
+}
+
 pub struct InternalGlContext<'a> {
     pub quad_context: &'a mut miniquad::Context,
     pub quad_gl: &'a mut crate::quad_gl::QuadGl,
